package com.mortals.xhx.module.h5;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.RepeatSubmit;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.utils.BeanUtil;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.datum.model.DatumInfoFieldEntity;
import com.mortals.xhx.module.datum.model.DatumInfoFieldQuery;
import com.mortals.xhx.module.datum.service.DatumInfoFieldService;
import com.mortals.xhx.module.h5.pdu.WechatInfo;
import com.mortals.xhx.module.h5.utils.WechatInfoUtil;
import com.mortals.xhx.module.home.pdu.HomeQueryPdu;
import com.mortals.xhx.module.hotword.model.HotwordEntity;
import com.mortals.xhx.module.hotword.model.HotwordQuery;
import com.mortals.xhx.module.hotword.service.HotwordService;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.user.model.SearchHistoryEntity;
import com.mortals.xhx.module.user.model.SearchHistoryQuery;
import com.mortals.xhx.module.user.model.UserFillHistoryEntity;
import com.mortals.xhx.module.user.service.SearchHistoryService;
import com.mortals.xhx.module.user.service.UserFillHistoryService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 微信小程序访问接口
 */
@RestController
@RequestMapping("wx/h5")
@Slf4j
public class WeChatMiniProgramController extends BaseJsonBodyController {

    /** php接口访问地址 */
    @Value("${server.php:http://192.168.0.98:8090}")
    private String phpServer;
    private static String ACTION = "/wechat/homebase/realInfo";
    @Autowired
    private HotwordService hotwordService;
    @Autowired
    private SearchHistoryService searchHistoryService;
    @Autowired
    private DatumInfoFieldService datumInfoFieldService;
    @Autowired
    private MatterService matterService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private UserFillHistoryService userFillHistoryService;

    /**
     * 首页数据，历史搜索和热搜
     * @param homeQueryPdu
     * @return
     */
    @PostMapping({"home/info"})
    @UnAuth
    public Rest<Object> h5HomePageInfo(@RequestBody HomeQueryPdu homeQueryPdu) {
        log.info("h5HomePageInfo:{}", JSON.toJSONString(homeQueryPdu));

        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "获取首页数据";
        int code = VALUE_RESULT_SUCCESS;
        try {
            HotwordQuery hotwordQuery = new HotwordQuery();
            hotwordQuery.siteId(homeQueryPdu.getSiteId());
            List<OrderCol> orderColList = new ArrayList<>();
            orderColList.add(new OrderCol("wordsSource", OrderCol.ASCENDING));
            orderColList.add(new OrderCol("searchCount", OrderCol.DESCENDING));
            hotwordQuery.setOrderColList(orderColList);

            PageInfo pageInfo = new PageInfo();
            int nums = GlobalSysInfo.getParamIntValue(Constant.PARAMS_HOTWORDS_DEFAULT_NUM, 10);
            pageInfo.setPrePageResult(nums);
            List<HotwordEntity> hotwordEntities = hotwordService.find(hotwordQuery, pageInfo, null).getList();
            model.put("hotWords", hotwordEntities);
            model.put("searchHistory", Collections.emptyList());
            try {
                String authtoken = request.getHeader("Authtoken");
                String phpUrl = phpServer + ACTION;
                WechatInfo wechatInfo = WechatInfoUtil.getWxInfo(phpUrl, authtoken);
                if(wechatInfo!=null){
                    SearchHistoryQuery historyQuery = new SearchHistoryQuery();
                    historyQuery.setOpenId(wechatInfo.getOpenid());
                    historyQuery.setMobile(wechatInfo.getEnphone());
                    historyQuery.setIdCard(wechatInfo.getEnidcard());
                    historyQuery.setOrderColList(Arrays.asList(new OrderCol("createTime", OrderCol.DESCENDING)));
                    pageInfo.setPrePageResult(5);
                    List<SearchHistoryEntity> historyEntities = searchHistoryService.find(historyQuery,pageInfo,null).getList();
                    model.put("searchHistory", historyEntities);
                }
            }catch (Exception e){
                log.info(e.getMessage());
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, var9);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        // log.info("home resp:{}", JSON.toJSONString(ret));
        return ret;
    }

    /**
     * 获取我的数字资产列表
     * @return
     */
    @PostMapping(value = "datum/info/field/list")
    @UnAuth
    public Rest<Object> fieldList() {
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询我的数字资产";
        try {
            DatumInfoFieldQuery query = new DatumInfoFieldQuery();
            String authtoken = request.getHeader("Authtoken");
            String phpUrl = phpServer + ACTION;
            WechatInfo wechatInfo = WechatInfoUtil.getWxInfo(phpUrl, authtoken);
            if(wechatInfo!=null){
                String openId = wechatInfo.getOpenid();
                String mobile = wechatInfo.getEnphone();
                String idCard = wechatInfo.getEnidcard();
                query.setOrConditionList(Arrays.asList(new DatumInfoFieldQuery().openId(openId),new DatumInfoFieldQuery().mobile(mobile),new DatumInfoFieldQuery().idCard(idCard)));
            }
            Rest<List<DatumInfoFieldEntity>> rest = datumInfoFieldService.distinctList(query, context);
            model.put(KEY_RESULT_DATA, rest.getData());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(request, busiDesc, model, e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(model);
    }

    /**
     * 修改我的数字资产列表
     * @return
     */
    @PostMapping({"datum/info/field/save"})
    @RepeatSubmit
    @UnAuth
    public Rest<Object> fieldSave(@RequestBody DatumInfoFieldEntity entity) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "修改我的数字资产";
        try {
            if(entity.getId()==null){
                throw new AppException("id不能为空");
            }
            entity.setUpdateTime(new Date());
            entity.setUpdateUserId(1L);
            this.datumInfoFieldService.update(entity, context);
            model.put("id", entity.getId());
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception e) {
            this.doException(this.request, busiDesc, model, e);
            return Rest.fail(e.getMessage());
        }
        return Rest.ok(model);
    }

    /**
     * 删除我的数字资产列表
     * @return
     */
    @PostMapping({"datum/info/field/delete"})
    @UnAuth
    public Rest<Object> h5Delete(@RequestBody DatumInfoFieldEntity entity) {
        Context context = this.getContext();
        if (entity.getId() == null) {
            return Rest.fail("请至少选择一条记录");
        } else {
            Map<String, Object> model = new HashMap();
            String busiDesc = "删除记录";
            try {
                int iRet = this.datumInfoFieldService.remove(entity.getId(), context);
                model.put("message_info",  "删除成功!删除条数:" + iRet);
                this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
            } catch (Exception e) {
                this.doException(this.request, busiDesc, model, e);
                return Rest.fail(e.getMessage());
            }
            return Rest.ok(model);
        }
    }

    /**
     * 小程序搜索事项或者材料
     * @param query
     * @return
     */
    @PostMapping({"home/search"})
    @UnAuth
    public Rest<Object> homeSearch(@RequestBody MatterEntity query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        int code=1;
        Context context = this.getContext();
        PageInfo pageInfo = this.buildPageInfo(query);
        String content = query.getMatterName();
        query.setMatterName("%" + content + "%");
        this.buildPermisionSql(query);
        Result<MatterEntity> result = this.matterService.find(query, pageInfo, context);
        model.put("matterList", result.getList());
        MatterDatumQuery matterDatumQuery = new MatterDatumQuery();
        matterDatumQuery.setMaterialName(query.getMatterName());
        Result<MatterDatumEntity> result1 = this.matterDatumService.find(matterDatumQuery, pageInfo, context);
        model.put("matterDatumList", result1.getList());
        try {
            String authtoken = request.getHeader("Authtoken");
            String phpUrl = phpServer + ACTION;
            WechatInfo wechatInfo = WechatInfoUtil.getWxInfo(phpUrl, authtoken);
            if(wechatInfo!=null){
                SearchHistoryEntity historyEntity = new SearchHistoryEntity();
                historyEntity.initAttrValue();
                historyEntity.setOpenId(wechatInfo.getOpenid());
                historyEntity.setMobile(wechatInfo.getEnphone());
                historyEntity.setIdCard(wechatInfo.getEnidcard());
                historyEntity.setContent(content);
                historyEntity.setCreateTime(new Date());
                historyEntity.setCreateUserId(1L);
                searchHistoryService.save(historyEntity);
            }
        }catch (Exception e){
            code=-1;
            log.info(e.getMessage());
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    protected PageInfo buildPageInfo(MatterEntity query) {
        PageInfo pageInfo = new PageInfo();
        if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getPage())) {
            pageInfo.setCurrPage(query.getPage());
        }

        if (!ObjectUtils.isEmpty(query) && !ObjectUtils.isEmpty(query.getSize())) {
            pageInfo.setPrePageResult(query.getSize());
        }

        if (ObjectUtils.isEmpty(query.getOrderColList())) {
            query.setOrderColList(Arrays.asList(new OrderCol("id", "desc")));
        }

        return pageInfo;
    }

    protected void buildPermisionSql(MatterEntity query) {
        String sql = (String)this.request.getAttribute("DATA_PERMISSION_SEARCH_SQL");
        if (!ObjectUtils.isEmpty(sql)) {
            query.setPermissionSql(sql);
        }

    }

    /**
     * 小程序保存填单信息
     * @param entity
     * @return
     */
    @PostMapping({"user/fill/history/save"})
    @RepeatSubmit
    @UnAuth
    public String saveFillHistory(@RequestBody UserFillHistoryEntity entity) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        int code = 1;
        String busiDesc = "微信小程序保存填单记录";
        try {
            if (entity.newEntity()) {
                UserFillHistoryEntity initEntity = new UserFillHistoryEntity();
                initEntity.initAttrValue();
                BeanUtils.copyProperties(entity, initEntity, BeanUtil.getNullPropertyNames(entity));
                entity = initEntity;
                initEntity.setCreateTime(new Date());
                initEntity.setCreateUserId(1L);
                String authtoken = request.getHeader("Authtoken");
                String phpUrl = phpServer + ACTION;
                WechatInfo wechatInfo = WechatInfoUtil.getWxInfo(phpUrl, authtoken);
                if(wechatInfo!=null){
                    initEntity.setOpenId(wechatInfo.getOpenid());
                    initEntity.setMobile(wechatInfo.getEnphone());
                    initEntity.setIdCard(wechatInfo.getEnidcard());
                }
                this.userFillHistoryService.save(initEntity, context);
            } else {
                entity.setUpdateTime(new Date());
                entity.setUpdateUserId(1L);
                this.userFillHistoryService.update(entity, context);
            }
            model.put("id", entity.getId());
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var9) {
            this.doException(this.request, busiDesc, model, var9);
            model.put("entity", entity);
            code = -1;
        }
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

    /**
     * 小程序获取事项材料详情
     * @param query
     * @return
     */
    @PostMapping({"matter/datum/info"})
    @UnAuth
    public String datumInfo(@RequestBody MatterDatumEntity query) {
        Map<String, Object> model = new HashMap();
        String busiDesc = "事项材料";
        if (query.getId() == null) {
            return this.createFailJsonResp("请选择待查看" + busiDesc + "信息");
        } else {
            JSONObject ret = new JSONObject();
            Context context = this.getContext();
            try {
                MatterDatumEntity entity = this.matterDatumService.get(query.getId(), context);
                if (entity == null) {
                    throw new Exception(busiDesc + "，不存在或已被删除");
                }
                ret.put("data", entity);
                List<MatterDatumEntity> datumEntityList = matterDatumService.find(new MatterDatumQuery().matterId(entity.getMatterId()));
                ret.put("datumList", datumEntityList);

                String authtoken = request.getHeader("Authtoken");
                String phpUrl = phpServer + ACTION;
                WechatInfo wechatInfo = WechatInfoUtil.getWxInfo(phpUrl, authtoken);
                if(wechatInfo!=null){
                    DatumInfoFieldQuery datumInfoFieldQuery = new DatumInfoFieldQuery();
                    datumInfoFieldQuery.setMatterId(entity.getMatterId());
                    datumInfoFieldQuery.setMaterialId(entity.getId());
                    String openId = wechatInfo.getOpenid();
                    String mobile = wechatInfo.getEnphone();
                    String idCard = wechatInfo.getEnidcard();
                    datumInfoFieldQuery.setOrConditionList(Arrays.asList(new DatumInfoFieldQuery().openId(openId),new DatumInfoFieldQuery().mobile(mobile),new DatumInfoFieldQuery().idCard(idCard)));
                    datumInfoFieldQuery.setOrderColList(Arrays.asList(new OrderCol("createTime", OrderCol.DESCENDING)));
                    List<DatumInfoFieldEntity> fieldEntities = datumInfoFieldService.find(datumInfoFieldQuery);
                    if(CollectionUtils.isNotEmpty(fieldEntities)){
                        Map<String,String> fieldMap = fieldEntities.stream().collect(Collectors.toMap(x -> x.getFieldCode(), y -> y.getFieldValue(), (o, n) -> n));
                        JSONObject formTemplate = JSONObject.parseObject(entity.getFormContent());
                        JSONArray formList = formTemplate.getJSONArray("list");
                        for (int j = 0; j < formList.size(); j++) {
                            JSONObject jsonObject = formList.getJSONObject(j);
                            String key = jsonObject.getString("id");
                            if(fieldMap.containsKey(key)){
                                jsonObject.put("value",fieldMap.get(key));
                            }
                        }
                        entity.setFormContent(formTemplate.toJSONString());
                        ret.put("data", entity);
                    }
                }
                if (!ObjectUtils.isEmpty(context) && !ObjectUtils.isEmpty(context.getUser())) {
                    this.recordSysLog(this.request, busiDesc + " 【成功】");
                }
            } catch (Exception var8) {
                this.doException(this.request, busiDesc, model, var8);
                Object msg = model.get("message_info");
                return this.createFailJsonResp(msg == null ? "系统异常" : msg.toString());
            }
            ret.put("code", 1);
            ret.put("msg", model.remove("message_info"));
            ret.put("dict", model.remove("dict"));
            return ret.toJSONString();
        }
    }
}
